#ifndef TFT_H
#define TFT_H  

#include "ILI9341_driver.h"
#include <device.h>
 
// Defines
#define CL_RED            	0xF800		// FF 00 00
#define CL_LIME           	0x07E0   	// 00 FF 00
#define CL_BLUE           	0x001F   	// 00 FF 00
#define CL_BLACK          	0x0000   	// 00 00 00
#define CL_WHITE          	0xFFFF   	// FF FF FF
#define CL_AQUA           	0x0FFF   	// 00 FF FF
#define CL_FUCHSIA        	0xF81F   	// FF 00 FF
#define CL_GRAY           	0x8410   	// 80 80 80
#define CL_GREEN          	0x0400   	// 00 80 00
#define CL_MAROON         	0x8000   	// 80 00 00
#define CL_NAVY           	0x0010   	// 00 00 80
#define CL_OLIVE          	0x8400   	// 80 80 00
#define CL_PURPLE         	0x8010   	// 80 00 80
#define CL_SILVER         	0xC618   	// C0 C0 C0
#define CL_TEAL           	0x0410   	// 00 80 80
#define CL_YELLOW         	0xFFE0   	// FF FF 00

#define FO_HORIZONTAL		0
#define FO_VERTICAL 		1

#define TOP_TO_BOTTOM 		0
#define LEFT_TO_RIGHT 		1

// Externs
extern uint16 TFT_DISP_WIDTH;
extern uint16 TFT_DISP_HEIGHT;

extern const uint8 TFT_defaultFont[2168];

/**
  * @brief  Moves "cursor" to the pointed location (usualy before writing on the display).
  * @param  _x : X position on the display.
  * @param  _y : Y position on the display.
  */
void TFT_Move_Cursor(uint16 _x, uint16 _y);

/**
  * @brief  Checks weather neccesary routines have been defined ()Set_Index, Write_Command and Write_Data.
  * @retval 0 - if procedures are not set. 
						1 - if procedures are set. 
  */
uint8 Is_TFT_Set();

/**
  * @brief  Defines pen properties e.g. lines etc.
  * @param  pen_color : Pen color.
  * @param  pen_width : Width of the pen (in pixels).
  */
void TFT_Set_Pen(uint16 pen_color, uint8 pen_width);

/**
  * @brief  Sets font to be used for all folowing char or text writing.
  * @param  activeFont : Pointer to font data array.
  * @param  font_color : Color of the font.
  * @param  font_orientation : Text orientation (FO_HORIZONTAL or FO_VERTICAL)
  */
void TFT_Set_Font(const uint8 *activeFont, uint16 font_color, uint8 font_orientation);

/**
  * @brief  Defines brush properties e.g. boxes, circles etc.
  * @param  brush_enabled : defines weather brush will be used or not.
  * @param  brush_color : brush color (not taken into account if gradient_enabled has been set).
  * @param  gradient_enabled : defines weather brush will have gradient (overrides color defined in brush_color).  
  * @param  gradient_orientation : top to bottom, bottom to top, left to right, right to left.    
  * @param  gradient_color_from : gradient start color.      
  * @param  gradient_color_to : gradient end color.        
  */
void TFT_Set_Brush(uint8 brush_enabled, uint16 brush_color, uint8 gradient_enabled, uint8 gradient_orientation, uint16 gradient_color_from, uint16 gradient_color_to);

/**
  * @brief  Converts RGB color (24bit) to 565 format (16bit)
  * @param  rgb_red : Red color intensity (0-255).
  * @param  rgb_green : Green color intensity (0-255).
  * @param  rgb_blue  : Blue color intensity (0-255).
  * @retval 16bit value rady to be used with TFT library. 
  */
uint16 TFT_RGBToColor16bit(uint8 rgb_red, uint8 rgb_green, uint8 rgb_blue);

/**
  * @brief  Converts 565 (16bit) color to 24bit RGB color format
  * @param  color : Color to be converted to RGB.  
  * @param  rgb_red : pointer to Red value.
  * @param  rgb_green : pointer to Green value.
  * @param  rgb_blue  : pointer to Blue value.
  */
void TFT_Color16bitToRGB(uint16 color, uint8 *rgb_red, uint8 *rgb_green, uint8 *rgb_blue);

/**
  * @brief  Writes single pixel dot on the display
  * @param  x : X coordinates.  
  * @param  y : Y coordinates.  
  * @param  color : color of the dot.
  */
void TFT_Dot(int16 x, int16 y, uint16 color);

/**
  * @brief  horizontal line on display.
  * @param  x_start : Start X position.
  * @param  x_end   : End X position.
  * @param  y_pos   : Y position
  */
void TFT_H_Line(int16 x_start, int16 x_end, int16 y_pos);

/**
  * @brief  Vertical line on display.
  * @param  y_start : Start Y position.
  * @param  y_end   : End Y position.
  * @param  x_pos   : X position
  */
void TFT_V_Line(int16 y_start, int16 y_end, int16 x_pos);

/**
  * @brief  Draws line on the display
  * @param  x1 : First point X position.
  * @param  y1 : First point Y position.
  * @param  x2 : Second point X position.
  * @param  y2 : Second point Y position.
  */
void TFT_Line(int16 x1, int16 y1, int16 x2, int16 y2);

/**
  * @brief  Fills entire screen with single color
  * @param  color : Color to fill the screen.
  */
void TFT_Fill_Screen(uint16 color);

/**
  * @brief  Writes circle on the display.
  * @param  x_center : Center X position of the circle
  * @param  y_center : Center Y position of the circle.
  * @param  radius   : Value of the circle's radius
  */
void TFT_Circle(int16 x_center, int16 y_center, int16 radius);

/**
  * @brief  Writes rectangle on the display.
  * @param  x_upper_left : Upper left X position
  * @param  y_upper_left : Upper left Y position
  * @param  x_bottom_right : Bottom right X position
  * @param  y_bottom_right : Bottom right Y position
  */
void TFT_Rectangle(int16 x_upper_left, int16 y_upper_left, int16 x_bottom_right, int16 y_bottom_right);

/**
  * @brief  Writes rectangle with round edges on the display.
  * @param  x_upper_left : Upper left X position
  * @param  y_upper_left : Upper left Y position
  * @param  x_bottom_right : Bottom right X position
  * @param  y_bottom_right : Bottom right Y position
  * @param  round_radius : Corner radius
  */
void TFT_Rectangle_Round_Edges(uint16 x_upper_left, uint16 y_upper_left, uint16 x_bottom_right, uint16 y_bottom_right, uint16 round_radius);

/**
  * @brief  Writes single character on display (with font previously set with TFT_Set_Font).
  * @param  ch : Character value (0-255)
  * @param  x : X position
  * @param  y : Y position
  */
void TFT_Write_uint8(uint16 ch, uint16 x, uint16 y);
void TFT_Write_Char(uint16 ch, uint16 x, uint16 y);

/**
  * @brief  Writes single character on display (with font previously set with TFT_Set_Font).
  * @param  ch : Character value (0-255)
  * @param  x : X position
  * @param  y : Y position
  */
void TFT_Write_Text(uint8 *text, uint16 x, uint16 y);
void TFT_Write_Text_Return_Pos( uint8 *text, uint16 x, uint16 y);

/**
  * @brief  Writes an bitmap image on display.
  * @param  left : Start X position of the image.
  * @param  top  : Start Y position of the image.
  * @param  image  : pointer to data array (containing image data)
  * @param  stretch : Strech factor (multiplier)  
  * @retval 0 - if image is loaded and displayed successfully. 
						1 - if error occured. 
  */
void TFT_Image(uint16 left, uint16 top, const  uint8 *image, uint8 stretch);

/**
  * @brief  Procedure writes portion of the bitmap image
  * @param  left: left coordinate of the image. 
  * @param  top: top coordinate of the image. 
  * @param  width: desired image width. 
  * @param  height: desired image height. 
  * @param  image: image to be displayed. Bitmap array is located in code memory. 
  * @param  stretch: stretches image by a given factor (if 2, it will double the image.). 
  */
void TFT_Partial_Image(uint16 left, uint16 top, uint16 width, uint16 height, const  uint8 *image, uint8 stretch);

/**
  * @brief  Set external font (from SD card etc)
  * @param  activeFont: offset where font data array is located on the SD card. 
  * @param  font_color: font color. 
  * @param  font_orientation : Text orientation (FO_HORIZONTAL or FO_VERTICAL)
  */
void TFT_Set_Ext_Font(uint32 activeFont, uint16 font_color, uint8 font_orientation);

/**
  * @brief  Write image from SD card
  * @param  left: X coordinates of the image. 
  * @param  top : Y coordinates of the image
  * @param  stretch: stretches image by a given factor (if 2, it will double the image.). 
  */
void TFT_Ext_Image(uint16 left, uint16 top, uint32 image, uint8 stretch);

/**
  * @brief  Write partial image from SD card
  * @param  left: left coordinate of the image. 
  * @param  top: top coordinate of the image. 
  * @param  width: desired image width. 
  * @param  height: desired image height. 
  * @param  image: offset where image data array is located. 
  * @param  stretch: stretches image by a given factor (if 2, it will double the image.). 
  */
void TFT_Ext_Partial_Image(uint16 left, uint16 top, uint16 width, uint16 height, uint32 image, uint8 stretch);

/**
  * @brief  Sets TFT in default mode. Must not be ommited in the code!
  */
void TFT_Set_Default_Mode();

/**
  * @brief  Initializes RAM buffer for external data.
  * @note	Must be called if external resurces are used.
  */
void TFT_Set_Ext_Buffer(uint8* (*getExtDataPtr)(uint32 offset, uint32 count, uint32* num));

#endif
//=================================================================================================
// End of File
//=================================================================================================
