/**************************************************************************************************
* File: MP3_driver.c
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2011-2011
* Description:
*     This module contains a set of functions that are used for communication with
*     VS1053E mp3 codec.
* NOTES:
*     Mp3 codec use SPI bus to communicate with MCU.
**************************************************************************************************/
#include <built_in.h>
#include "MP3_driver.h"

void SPI_Write_Byte(uint8 _data) {
    emFile_SPI0_WriteTxData(_data);
    
    while(!(emFile_SPI0_STS_SPI_DONE & emFile_SPI0_ReadTxStatus()));
    
    emFile_SPI0_ClearRxBuffer();
    emFile_SPI0_ClearTxBuffer();
    
    return;
}

uint8 SPI_Read_Byte(void) {
    uint8 _temp;
    
    emFile_SPI0_WriteTxData(0);
    
    while(!(emFile_SPI0_STS_SPI_DONE & emFile_SPI0_ReadTxStatus()));

    _temp = emFile_SPI0_ReadRxData();
    
    emFile_SPI0_ClearRxBuffer();
    emFile_SPI0_ClearTxBuffer();
    
    return _temp;
}

/**************************************************************************************************
* Function MP3_SCI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function writes one byte to MP3 SCI
* Input: register address in codec, data
* Output: Nothing
**************************************************************************************************/
void MP3_SCI_Write(uint8 address, uint16 data_in) {
  MP3_BSYNC1;

  MP3_CS0;                          // select MP3 SCI
  SPI_Write_Byte(WRITE_CODE);
  SPI_Write_Byte(address);
  SPI_Write_Byte(Hi(data_in));           // high byte
  SPI_Write_Byte(Lo(data_in));           // low byte
  MP3_CS1;                          // deselect MP3 SCI
  while (MP3_DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SCI_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads words_count words from MP3 SCI
* Input: start address, word count to be read
* Output: words are stored to data_buffer
**************************************************************************************************/
void MP3_SCI_Read(uint8 start_address, uint8 words_count, uint16 *data_buffer) {
  uint16 temp;

  MP3_CS0;                    // select MP3 SCI
  SPI_Write_Byte(READ_CODE);
  SPI_Write_Byte(start_address);

  while (words_count--) {        // read words_count words byte per byte
    temp = SPI_Read_Byte();
    temp <<= 8;
    temp += SPI_Read_Byte();
    *(data_buffer++) = temp;
  }
  MP3_CS1;                    // deselect MP3 SCI
  while (MP3_DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SDI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write one byte to MP3 SDI
* Input: data to be writed
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write(uint8 data_) {

  MP3_CS1;
  MP3_BSYNC0;

  while (MP3_DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI_Write_Byte(data_);
  MP3_BSYNC1;
}

/**************************************************************************************************
* Function MP3_SDI_Write_32
* -------------------------------------------------------------------------------------------------
* Overview: Function Write 32 bytes to MP3 SDI
* Input: data buffer
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write_32(uint8 *data_) {
  uint8 i;

  MP3_CS1;
  MP3_BSYNC0;

  while (MP3_DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI_Write_Byte(data_[i]);
  MP3_BSYNC1;
}

/**************************************************************************************************
* Function MP3_Set_Volume()
* -------------------------------------------------------------------------------------------------
* Overview: Function set volume on the left and right channel
* Input: left channel volume, right channel volume
* Output: Nothing
**************************************************************************************************/
void MP3_Set_Volume(uint8 left, uint8 right) {
  uint16 volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Init() {
  uint8 volume_left, volume_right;
  
  MP3_CS1;
  MP3_BSYNC1;
  
//  BSYNC = 1;
//  MP3_CS = 1;

  // Hardware reset
  MP3_RST0;
  CyDelay(10);
  MP3_RST1;

  while (MP3_DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}
/**************************************************************************************************
* End of File
**************************************************************************************************/